package dev.peerat.framework;

import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.function.Consumer;
import java.util.function.Supplier;

public class Locker<V>{
	
	private Map<Key, BlockingQueue<V>> map;
	
	public Locker(){
		this.map = new ConcurrentHashMap<>();
	}
	
	public void init(Key key){
		this.map.put(key, new LinkedBlockingQueue<>());
	}
	
	public void remove(Key key){
		this.map.remove(key);
	}
	
	private BlockingQueue<V> get(Key key){
		return this.map.get(key);
	}
	
	public void pushValue(V value){
		for(BlockingQueue<V> queue : this.map.values()){
			queue.add(value);
			unlock(queue);
		}
	}
	
	public V getValue(Key key) throws InterruptedException{
		BlockingQueue<V> queue = get(key);
		lock(queue);
		return queue.poll();
	}

	private void lock(BlockingQueue<V> queue) throws InterruptedException{
		if(queue.isEmpty()){
			synchronized(queue){
				queue.wait();
			}
		}
	}
	
	private void unlock(BlockingQueue<V> queue){
		synchronized(queue){
			queue.notify();
		}
	}
	
	public void listen(Consumer<V> action){
		Key key = new Key();
		init(key);
		try {
			while(true) action.accept(getValue(key));
		}catch(Exception e){}
		remove(key);
	}
	
	public void listen(Consumer<V> action, Consumer<Exception> onClose){
		Key key = new Key();
		init(key);
		try {
			while(true) action.accept(getValue(key));
		}catch(Exception e){
			onClose.accept(e);
		}
		remove(key);
	}
	
	public void listen(Supplier<Boolean> condition, Consumer<V> action){
		Key key = new Key();
		init(key);
		try {
			while(condition.get()) action.accept(getValue(key));
		}catch(Exception e){}
		remove(key);
	}
	
	public void listen(Supplier<Boolean> condition, Consumer<V> action, Consumer<Exception> onClose){
		Key key = new Key();
		init(key);
		try {
			while(condition.get()) action.accept(getValue(key));
		}catch(Exception e){
			onClose.accept(e);
		}
		remove(key);
	}
	
	public static class Key{ public Key(){} }
}
